/////////////////////////////////////////////////////////////////////////////////////////
// 
//                 Université de Nice Sophia-Antipolis  (UNS) - 
//                 Centre National de la Recherche Scientifique (CNRS)
//                 Copyright © 2015 UNS, CNRS All Rights Reserved.
// 
//     These computer program listings and specifications, herein, are
//     the property of Université de Nice Sophia-Antipolis and CNRS
//     shall not be reproduced or copied or used in whole or in part as
//     the basis for manufacture or sale of items without written permission.
//     For a license agreement, please contact:
//     <mailto: licensing@sattse.com> 
//
//
//
//     Author: Luc Hogie – Laboratoire I3S - luc.hogie@unice.fr
//
//////////////////////////////////////////////////////////////////////////////////////////

package jacaboo;

import java.io.IOException;
import java.io.ObjectInput;
import java.io.ObjectOutput;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.List;

import toools.text.LineStreamListener;

public class SSHNode extends HardwareNode
{
	private final List<LineStreamListener> stdoutLinesListeners = new ArrayList<>();
	private final List<LineStreamListener> stderrLinesListeners = new ArrayList<>();
	private String loginName;
	public OutputStream stdin;
	public RemoteMain main;
	public Process process;
	public JVM jvm;

	/**
	 * For serialization. Do not use.
	 */
	public SSHNode()
	{
	}

	public SSHNode(String nodeSpec) throws UnknownHostException
	{
		super();
		fromString(nodeSpec, System.getProperty("user.name"));
	}

	public SSHNode(String nodeSpec, String defaultUsername) throws UnknownHostException
	{
		super();
		fromString(nodeSpec, defaultUsername);
	}

	protected void fromString(String nodeSpec, String defaultUsername)
			throws UnknownHostException
	{
		String comps[] = nodeSpec.split("@");

		switch (comps.length)
		{
		case 1:
			super.fromString(nodeSpec);
			this.loginName = defaultUsername;
			break;
		case 2:
			super.fromString(comps[1]);
			this.loginName = comps[0];
			break;
		default:
			throw new UnknownHostException(nodeSpec);
		}

		stdoutLinesListeners.add(new ToStreamLineReceiver(System.out, "> "
				+ this.toString() + ": "));
		stderrLinesListeners.add(new ToStreamLineReceiver(System.err, "> "
				+ this.toString() + ": "));
	}

	protected SSHNode(InetAddress addr)
	{
		super(addr);
		this.loginName = System.getProperty("user.name");
		stdoutLinesListeners.add(new ToStreamLineReceiver(System.out, "> "
				+ this.toString() + ": "));
		stderrLinesListeners.add(new ToStreamLineReceiver(System.err, "> "
				+ this.toString() + ": "));
	}

	public String getLoginName()
	{
		return loginName;
	}

	public String getSSHName()
	{
		if (loginName == null)
		{
			return getPublicInetAddress().getHostName();
		}
		else
		{
			return loginName + "@" + getPublicInetAddress().getHostName();
		}
	}

	@Override
	public String toString()
	{
		return super.toString();
	}

	public List<LineStreamListener> getStdoutLinesListeners()
	{
		return stdoutLinesListeners;
	}

	public List<LineStreamListener> getStderrLinesListeners()
	{
		return stderrLinesListeners;
	}

	@Override
	public void writeExternal(ObjectOutput out) throws IOException
	{
		super.writeExternal(out);
		out.writeUTF(getLoginName());
	}

	@Override
	public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException
	{
		super.readExternal(in);
		loginName = in.readUTF();
	}

	public void kill_9_1()
	{
		SSHUtils.execSh(2000, this, "kill -9 - 1");
	}
}
